package PerlWebmail::UserConfig;
use vars qw( $VERSION $NAME);
use strict;
use warnings;
$NAME = 'PerlWebmail::UserConfig';
$VERSION = '3.0.4';

sub new {
  my $class = shift;
  my $self = {
    debug => undef,
    initialized => undef,
    config => {},
  };
  my %args = @_;
  if( $args{-user_storage_dir} && $args{-email} ){
    require PerlWebmail::UserConfig::File;
    bless $self,'PerlWebmail::UserConfig::File';
  }elsif( $args{-dbstring} && $args{-dbuser} && $args{-dbpass} && $args{-email} ){
    require PerlWebmail::UserConfig::DB;
    bless $self,'PerlWebmail::UserConfig::DB';
  }else{
    bless $self,$class;
  }
  return $self if $self->init(@_);
}

# stub
sub init {
  my $self = shift;
  # do not set self->{initialized}
}

# stub
sub read_config {
  my $self = shift;
}

# stub
sub write_config {
  my $self = shift;
}

sub config {
  my $self = shift;
  warn ref($self) . " config: called\n" if $self->{debug};
  return unless ref($self);
  if( my $config = shift ){
    $self->{config} = $config;
    $self->write_config();
  }
  $self->{config};
}

sub set {
  my $self = shift;
  warn ref($self) . " set: called\n" if $self->{debug};
  my (%args) = @_;
  map { $self->{config}->{$_} = $args{$_} } keys %args;
}

sub addfolder {
  my $self = shift;
  warn ref($self) . " addfolder: called\n" if $self->{debug};
  my $folder = shift;
  return unless $folder;
  push(@{$self->{config}->{folders}},$folder);
  $self->write_config();
}

sub listfolders {
  my $self = shift;
  @{ $self->{config}->{folders} } if $self->{config}->{folders};
}

sub removefolder {
  my $self = shift;
  my $folder = shift;
  return unless $folder;
  my $tmp = [];
  foreach my $i ( @{ $self->{config}->{folders} } ){
    push(@$tmp,$i) unless ($i eq $folder);
  }
  $self->{config}->{folders} = $tmp;
  $self->write_config();
}

sub setnote {
  my $self = shift;
  warn ref($self) . " setnote: called\n" if $self->{debug};
  my (%args) = @_;
  return unless ( $args{-year} && $args{-month} && $args{-day} && $args{-note} );
  $self->{config}->{notes}->{ $args{-year} }->{ $args{-month} }->{ $args{-day} } = $args{-note};
  $self->write_config();
}

sub getnote {
  my $self = shift;
  warn ref($self) . " getnote: called\n" if $self->{debug};
  my (%args) = @_;
  return unless ( $args{-year} && $args{-month} && $args{-day} );
  $self->{config}->{notes}->{ $args{-year} }->{ $args{-month} }->{ $args{-day} };
}

sub removenote {
  my $self = shift;
  warn ref($self) . " removenote: called\n" if $self->{debug};
  my (%args) = @_;
  return unless ( $args{-year} && $args{-month} && $args{-day} );
  delete $self->{config}->{notes}->{ $args{-year} }->{ $args{-month} }->{ $args{-day} };
  $self->write_config();
}

sub setcontact {
  my $self = shift;
  warn ref($self) . " setcontact: called\n" if $self->{debug};
  my (%args) = @_;
  return unless ( $args{-email} );

  ### pull out the email and remove it from args hash
  my $email = $args{-email};
  delete $args{-email};

  ### use email as key, set rest of args as value
  $self->{config}->{contacts}->{ $email } = { %args };
  $self->write_config();
}

sub listcontacts {
  my $self = shift;
  warn ref($self) . " listcontacts: called\n" if $self->{debug};
  %{ $self->{config}->{contacts} } if $self->{config}->{contacts};
}

sub removecontact {
  my $self = shift;
  warn ref($self) . " removecontact: called\n" if $self->{debug};
  my (%args) = @_;
  return unless ( $args{-email} );

  delete $self->{config}->{contacts}->{ $args{-email} };
  $self->write_config();
}

1;

__END__

=head1 NAME

PerlWebmail::UserConfig - User Configuration object (base class)

=head1 SYNOPSIS

 # uses ::File or ::DB depending on arguments passed in
 use PerlWebmail::UserConfig;

 my $config_obj = new PerlWebmail::UserConfig(

  # for file based
  -user_storage_dir => '/var/lib/perlwebmail',
  -email => 'johndoe@publicdomain.com',

  # or

  # for db based
  -dbstring => 'DBI:Pg:dbname=DBNAME',
  -dbuser => 'DBUSER',
  -dbpass => 'DBPASS',
  -email => 'user@localhost',

 );

 ### retrieve config 
 $hashref_config = $config_obj->config();
 ### set config using external datasource
 $config_obj->config( $hashref_config );
 ### set config option
 $config_obj->set( key => value );

 ### add and remove folders
 $config_obj->addfolder( 'folder_name' );
 $config_obj->removefolder( 'folder_name' );
 ### list folders
 $config_obj->listfolders();

 ### add, retrieve, and remove notes
 $config_obj->setnote( -year => 2002, -month => 1, -day => 26, -note => 'Happy Birthday' );
 my $note_string = $config_obj->getnote( -year => 2002, -month => 1, -day => 26 );
 $config_obj->removenote( -year => 2002, -month => 1, -day => 26 );

 ### add, list, and remove contacts
 $config_obj->setcontact( -email => 'johndoe@publicdomain.com', attribute1 => 'value', attribute2 => 'value' );
 my %contacts = $config_obj->listcontacts();
 $config_obj->removecontact( -email => 'johndoe@publicdomain.com' );

=head1 CONSTRUCTOR

=over 4

=item I<-debug>

 Description: turns debugging on or off [default off].

=item I<-user_storage_dir>

 Description:  Base storage directory

=item I<-dbstring>

 Description:  Database location string

=item I<-dbuser>

 Description:  Database user name

=item I<-dbpass>

 Description:  Database password

=item I<-email>

 Description: email address (used for storage folder)

=back

=head1 METH0DS

=over 4

=item I<new> Constructor

 Description: 

=item I<config> Returns config as hashref

 Description: 

=item I<read_config>

 Description: Commits config to storage
 Arguments: none

=item I<write_config>

 Description: Commits config to storage
 Arguments: none

=item I<set>

 Description: 

=item I<addfolder>

 Description: Adds folder to internal folder list
 Arguments: 'folder name'

=item I<removefolder>

 Description: Removes folder to internal folder list
 Arguments: 'folder name'

=item I<setnote>

 Description: Adds note for specified year, month, and day
 Arguments: -year =>, -month =>, -day =>, -note =>

=item I<getnote>

 Description: Retrieves note for specified year, month, and day
 Arguments: -year =>, -month =>, -day =>, -note =>

=item I<removenote>

 Description: Removes note for specified year, month, and day
 Arguments: -year =>, -month =>, -day =>

=item I<setcontact>

 Description: Adds a contact 
 Arguments: -email => '', attribute_keys => values ..

=item I<listcontacts>

 Description: returns hash of contacts
 Arguments: none

=item I<removecontact>

 Description: Removes contact
 Arguments: -email => ''

=back

=head1 AUTHOR

 Jason Woodward <woodwardj@jaos.org>

=cut
