package PerlWebmail::UserConfig::DB;
use base PerlWebmail::UserConfig;
use vars qw( $VERSION $NAME $dbo $serializer );
use strict;
use warnings;
use DBI;
use Digest::MD5  qw(md5 md5_hex md5_base64);
use Data::Serializer;
$NAME = 'PerlWebmail::UserConfig::DB';
$VERSION = '3.0.6';

sub init {
  my $self = shift;
  return 0 unless ref($self) eq 'PerlWebmail::UserConfig::DB';
  my (%args) = @_;

  $self->{debug} = $args{-debug} if $args{-debug};
  warn ref($self) . " init: called\n" if $self->{debug};

  die "dbstring and email not specified" unless ( $args{-dbstring} && $args{-email} );

  $self->{dbstring} = $args{-dbstring};
  $self->{dbuser} = $args{-dbuser} if $args{-dbuser};
  $self->{dbpass} = $args{-dbpass} if $args{-dbpass};
  $self->{dbattr} = $args{-dbattr} if $args{-dbattr};
  $self->{email} = $args{-email};

  ### init db here
  $dbo = DBI->connect($self->{dbstring},$self->{dbuser},$self->{dbpass},$self->{dbattr}) unless $dbo;
  $serializer = Data::Serializer->new(
    serializer => 'Data::Dumper',
    digester   => 'MD5',
    cipher     => 'Blowfish',
    secret     => undef,
    compress   => '1',
  ) unless $serializer;
  die "Cannot open database or create serializer" unless ( $dbo && $serializer );

  ### try to get REMOTE_ADDR
  $self->{remote_addr} = ( %ENV && $ENV{REMOTE_ADDR} ) ? $ENV{REMOTE_ADDR} : '';

  $self->read_config();

  unless ( defined $self->{last_accessed} ){
    die "Cannot create db entry" unless $self->create_db_entry();
  }
  return 1;
}

### create a session entry
sub create_db_entry {
  my $self = shift;
  return unless ref($self) eq 'PerlWebmail::UserConfig::DB';
  warn ref($self) . " create_db_entry: called\n" if $self->{debug};
  return unless $self->{email};

  my $rc = $dbo->do("insert into webmail_userconfig values ('$self->{email}','$self->{remote_addr}','',now())");
  warn $DBI::errstr,"\n" if $self->{debug} && $DBI::errstr;
  return $rc;
}

### read the session entry
sub read_config {
  my $self = shift;
  return unless ref($self) eq 'PerlWebmail::UserConfig::DB';
  warn ref($self) . " read_config: called\n" if $self->{debug};
  return unless $self->{email};

  my $stmt = $dbo->prepare_cached('select data,remote_addr,last_accessed from webmail_userconfig where email = ?');
  $stmt->execute( $self->{email} );

  warn $DBI::errstr,"\n" if $self->{debug} && $DBI::errstr;

  if( my ($data,$remote_addr,$last_accessed) = $stmt->fetchrow_array ){
    $self->{config} = $serializer->deserialize( $data );
    $self->{remote_addr} = $remote_addr;
    $self->{last_accessed} = $last_accessed;
  }

}

### write the session config data payload
sub write_config {
  my $self = shift;
  return unless ref($self) eq 'PerlWebmail::UserConfig::DB';
  warn ref($self) . " write_config: called\n" if $self->{debug};
  return unless $self->{email};

  my $sql = "update webmail_userconfig set data = ?,last_accessed = now(), remote_addr = ? where email = ?";
  my $update = $dbo->prepare_cached($sql);
  $update->execute(
    $serializer->serialize( $self->{config} ),
    $self->{remote_addr},
    $self->{email}
  );

  warn $DBI::errstr,"\n" if $self->{debug} && $DBI::errstr;

}

1;

__END__

=head1 NAME

PerlWebmail::UserConfig::DB - User Configuration object with DB based data store

=head1 SYNOPSIS

 ### requires db table like this (pg centric):
 create table webmail_userconfig (
  email varchar(255) PRIMARY KEY,
  remote_addr varchar(80),
  data text ,
  last_accessed timestamp default now()
 );


 ### you can call explicitly or via PerlWebmail::UserConfig
 use PerlWebmail::UserConfig;
 # or
 use PerlWebmail::UserConfig::DB;

 my $config_obj = new PerlWebmail::UserConfig::DB(
  -dbstring => 'DBI:Pg:dbname=DBNAME',
  -dbuser => 'DBUSER',
  -dbpass => 'DBPASS',
  -email => 'user@localhost',
  -debug => 1,
 );

 ### retrieve config 
 $hashref_config = $config_obj->config();
 ### set config using external datasource
 $config_obj->config( $hashref_config );
 ### set config option
 $config_obj->set( key => value );

 ### add and remove folders
 $config_obj->addfolder( 'folder_name' );
 $config_obj->removefolder( 'folder_name' );
 ### list folders
 $config_obj->listfolders();

 ### add, retrieve, and remove notes
 $config_obj->setnote( -year => 2002, -month => 1, -day => 26, -note => 'Happy Birthday' );
 my $note_string = $config_obj->getnote( -year => 2002, -month => 1, -day => 26 );
 $config_obj->removenote( -year => 2002, -month => 1, -day => 26 );

 ### add, list, and remove contacts
 $config_obj->setcontact( -email => 'johndoe@publicdomain.com', attribute1 => 'value', attribute2 => 'value' );
 my %contacts = $config_obj->listcontacts();
 $config_obj->removecontact( -email => 'johndoe@publicdomain.com' );

=head1 CONSTRUCTOR

=over 4

=item I<-debug>

 Description: turns debugging on or off [default off].

=item I<-dbstring>

 Description:  Database location string

=item I<-dbuser>

 Description:  Database user name

=item I<-dbpass>

 Description:  Database password

=item I<-email>

 Description: email address (used for storage folder)

=back

=head1 METH0DS

=over 4

=item I<new> Constructor

 Description: 

=item I<config> Returns config as hashref

 Description: 

=item I<read_config>

 Description: Commits config to storage
 Arguments: none

=item I<write_config>

 Description: Commits config to storage
 Arguments: none

=item I<set>

 Description: 

=item I<addfolder>

 Description: Adds folder to internal folder list
 Arguments: 'folder name'

=item I<removefolder>

 Description: Removes folder to internal folder list
 Arguments: 'folder name'

=item I<setnote>

 Description: Adds note for specified year, month, and day
 Arguments: -year =>, -month =>, -day =>, -note =>

=item I<getnote>

 Description: Retrieves note for specified year, month, and day
 Arguments: -year =>, -month =>, -day =>, -note =>

=item I<removenote>

 Description: Removes note for specified year, month, and day
 Arguments: -year =>, -month =>, -day =>

=item I<setcontact>

 Description: Adds a contact 
 Arguments: -email => '', attribute_keys => values ..

=item I<listcontacts>

 Description: returns hash of contacts
 Arguments: none

=item I<removecontact>

 Description: Removes contact
 Arguments: -email => ''

=back

=head1 AUTHOR

 Jason Woodward <woodwardj@jaos.org>

=cut
